/**
 * @author pastakhov, MusikAnimal and others
 * @license GPL-2.0-or-later
 * @see https://gerrit.wikimedia.org/g/mediawiki/extensions/CodeMirror
 */
import { LanguageSupport } from '@codemirror/language';
import { Tag } from '@lezer/highlight';
import type { StreamParser, StringStream, TagStyle } from '@codemirror/language';
import type { CompletionSource, Completion } from '@codemirror/autocomplete';
declare const tokens: {
    apostrophes: string;
    comment: string;
    doubleUnderscore: string;
    em: string;
    error: string;
    extLink: string;
    extLinkBracket: string;
    extLinkProtocol: string;
    extLinkText: string;
    extTag: string;
    extTagAttribute: string;
    extTagBracket: string;
    extTagName: string;
    freeExtLink: string;
    freeExtLinkProtocol: string;
    hr: string;
    htmlEntity: string;
    htmlTagAttribute: string;
    htmlTagBracket: string;
    htmlTagName: string;
    imageParameter: string;
    linkBracket: string;
    linkDelimiter: string;
    linkPageName: string;
    linkText: string;
    linkToSection: string;
    list: string;
    pageName: string;
    parserFunction: string;
    parserFunctionBracket: string;
    parserFunctionDelimiter: string;
    parserFunctionName: string;
    redirect: string;
    section: string;
    sectionHeader: string;
    signature: string;
    skipFormatting: string;
    strong: string;
    tableBracket: string;
    tableCaption: string;
    tableDefinition: string;
    tableDelimiter: string;
    tableDelimiter2: string;
    template: string;
    templateArgumentName: string;
    templateBracket: string;
    templateDelimiter: string;
    templateName: string;
    templateVariable: string;
    templateVariableBracket: string;
    templateVariableDelimiter: string;
    templateVariableName: string;
};
declare type Style = string | [string];
declare type Tokenizer<T = Style> = (stream: StringStream, state: State) => T;
declare type TagName = keyof typeof tokens;
declare type NestCount = 'nTemplate' | 'nExt' | 'nVar' | 'nLink' | 'nExtLink';
declare interface Nesting extends Record<NestCount, number> {
    extName: string | false;
    extState: object | false;
}
declare interface State extends Nesting {
    tokenize: Tokenizer;
    readonly stack: Tokenizer[];
    readonly inHtmlTag: string[];
    extMode: StreamParser<object> | false;
    lbrack: boolean | undefined;
    bold: boolean;
    italic: boolean;
    dt: Partial<Nesting> & {
        n: number;
    };
    redirect: boolean;
}
declare type ExtState = Omit<State, 'dt'> & Partial<Pick<State, 'dt'>>;
declare interface Token {
    pos: number;
    readonly string: string;
    style: Style;
    readonly state: State;
}
export type ApiSuggestions = [string, string?][];
/**
 * 获取维基链接建议
 * @param search 搜索字符串，开头不包含` `
 * @param namespace 命名空间
 * @param subpage 是否为子页面
 */
export type ApiSuggest = (search: string, namespace?: number, subpage?: boolean) => ApiSuggestions | Promise<ApiSuggestions>;
export interface MwConfig {
    readonly tags: Record<string, true>;
    readonly tagModes: Record<string, string>;
    urlProtocols: string;
    functionSynonyms: [Record<string, string>, Record<string, unknown>];
    doubleUnderscore: [Record<string, unknown>, Record<string, unknown>];
    nsid: Record<string, number>;
    variants?: string[];
    img?: Record<string, string>;
    redirection?: string[];
    permittedHtmlTags?: string[];
    implicitlyClosedHtmlTags?: string[];
    linkSuggest?: ApiSuggest;
    paramSuggest?: ApiSuggest;
}
declare const enum TableCell {
    Td = 0,
    Th = 1,
    Caption = 2
}
/** Adapted from the original CodeMirror 5 stream parser by Pavel Astakhov */
export declare class MediaWiki {
    #private;
    /** 已解析的节点 */
    readonly readyTokens: Token[];
    /** 当前起始位置 */
    oldToken: Token | null;
    /** 可能需要回滚的`'''` */
    mark: number | null;
    readonly config: MwConfig;
    firstSingleLetterWord: number | null;
    firstMultiLetterWord: number | null;
    firstSpace: number | null;
    readonly tokenTable: {
        [x: string]: Tag;
    };
    readonly hiddenTable: Record<string, Tag>;
    readonly permittedHtmlTags: Set<string>;
    readonly implicitlyClosedHtmlTags: Set<string>;
    readonly urlProtocols: RegExp;
    readonly fileRegex: RegExp;
    readonly nsRegex: RegExp;
    readonly redirectRegex: RegExp;
    readonly imgRegex: RegExp;
    readonly functionSynonyms: Completion[];
    readonly doubleUnderscore: Completion[];
    readonly extTags: Completion[];
    readonly htmlTags: Completion[];
    readonly protocols: Completion[];
    readonly imgKeys: Completion[];
    readonly htmlAttrs: Completion[];
    readonly elementAttrs: Map<string | undefined, Completion[]>;
    readonly extAttrs: Map<string, Completion[]>;
    constructor(config: MwConfig);
    /**
     * Dynamically register a token in CodeMirror.
     * This is solely for use by this.addTag() and CodeMirrorModeMediaWiki.makeLocalStyle().
     *
     * @param token
     * @param hidden Whether the token is not highlighted
     * @param parent
     */
    addToken(token: string, hidden?: boolean, parent?: Tag): void;
    /**
     * Register the ground tokens. These aren't referenced directly in the StreamParser, nor do
     * they have a parent Tag, so we don't need them as constants like we do for other tokens.
     * See this.makeLocalStyle() for how these tokens are used.
     */
    registerGroundTokens(): void;
    /**
     * This defines the actual CSS class assigned to each tag/token.
     *
     * @see https://codemirror.net/docs/ref/#language.TagStyle
     */
    getTagStyles(): TagStyle[];
    makeFullStyle(style: Style, state: ExtState): string;
    makeTagStyle(tag: TagName, state: State, endGround?: NestCount): [string];
    makeStyle(style: string, state: ExtState, endGround?: NestCount): [string];
    makeLocalTagStyle(tag: TagName, state: State, endGround?: NestCount): string;
    makeLocalStyle(style: string, state: ExtState, endGround?: NestCount): string;
    inStr(str: string, tag: TagName | false, errorTag?: TagName): Tokenizer;
    /**
     * @todo 添加stage参数
     * @ignore
     */
    eatWikiText(style: string): Tokenizer;
    eatApostrophes(obj: Pick<State, 'bold' | 'italic'>): Tokenizer<string | false>;
    eatExternalLinkProtocol(chars: string, free?: boolean): Tokenizer;
    inExternalLink(text?: boolean): Tokenizer;
    get eatFreeExternalLink(): Tokenizer;
    inLink(file: boolean, section?: boolean): Tokenizer;
    inLinkText(file: boolean): Tokenizer;
    toEatImageParameter(stream: StringStream, state: State): void;
    inSectionHeader(count: number): Tokenizer;
    get eatSectionHeader(): Tokenizer;
    eatList(stream: StringStream, state: State): string;
    get eatStartTable(): Tokenizer;
    inTableDefinition(tr?: boolean): Tokenizer;
    get inTable(): Tokenizer;
    inTableCell(needAttr: boolean, type: TableCell, firstLine?: boolean): Tokenizer;
    eatEntity(stream: StringStream, style: string): string;
    get inVariable(): Tokenizer;
    inVariableDefault(isFirst?: boolean): Tokenizer;
    get inComment(): Tokenizer;
    inParserFunctionName(sameLine?: boolean, invoke?: boolean): Tokenizer;
    inParserFunctionArguments(module?: boolean): Tokenizer;
    inTemplatePageName(haveEaten?: boolean, anchor?: boolean): Tokenizer;
    inTemplateArgument(expectName?: boolean): Tokenizer;
    eatTagName(chars: string, isCloseTag?: boolean, isHtmlTag?: boolean): Tokenizer;
    inHtmlTagAttribute(name: string): Tokenizer;
    get eatNowiki(): Tokenizer<string>;
    inExtTagAttribute(name: string): Tokenizer;
    eatExtTagArea(name: string): Tokenizer;
    eatExtCloseTag(name: string): Tokenizer;
    inExtTokens(origString: string | false): Tokenizer;
    /**
     * Remembers position and status for rollbacking.
     * It is needed for changing from bold to italic with apostrophes before it, if required.
     *
     * @see https://phabricator.wikimedia.org/T108455
     * @param stream
     */
    prepareItalicForCorrection(stream: StringStream): void;
    /**
     * main entry
     *
     * @see https://codemirror.net/docs/ref/#language.StreamParser
     */
    get mediawiki(): StreamParser<State>;
    get 'text/mediawiki'(): StreamParser<State>;
    /** 自动补全魔术字和标签名 */
    get completionSource(): CompletionSource;
}
/**
 * Gets a LanguageSupport instance for the MediaWiki mode.
 * @param config Configuration for the MediaWiki mode
 */
export declare const mediawiki: (config: MwConfig) => LanguageSupport;
/**
 * Gets a LanguageSupport instance for the mixed MediaWiki-HTML mode.
 * @param config Configuration for the MediaWiki mode
 */
export declare const html: (config: MwConfig) => LanguageSupport;
export {};
